import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JPasswordField;

import SecureBlackbox.Base.*;
import SecureBlackbox.Cloud.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class UnprotHandlerPropsDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextArea tbEncryptionCerts;
	private JTextField tbCertPath;
	private JPasswordField tbCertPassword;
	private JTextArea tbProtProps;

    private TElDefaultDataStorageSecurityHandler m_Handler = null;
    private TElMemoryCertStorage m_EncryptionCerts = new TElMemoryCertStorage();

	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UnprotHandlerPropsDialog dialog = new UnprotHandlerPropsDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public UnprotHandlerPropsDialog() {
		setTitle("Object protection settings");
		setBounds(100, 100, 469, 390);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			JLabel lblObjectIsEncrypted = new JLabel("Object is encrypted for the following certificates:");
			lblObjectIsEncrypted.setBounds(10, 11, 398, 14);
			contentPanel.add(lblObjectIsEncrypted);
		}
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 32, 442, 120);
		contentPanel.add(scrollPane);
		
		tbEncryptionCerts = new JTextArea();
		tbEncryptionCerts.setEnabled(false);
		tbEncryptionCerts.setLineWrap(true);
		tbEncryptionCerts.setWrapStyleWord(true);
		scrollPane.setViewportView(tbEncryptionCerts);
		
		JLabel lblPleaseSpecifyThe = new JLabel("Please specify the file containing certificate with the corresponding private key:");
		lblPleaseSpecifyThe.setBounds(10, 163, 442, 14);
		contentPanel.add(lblPleaseSpecifyThe);
		
		tbCertPath = new JTextField();
		tbCertPath.setBounds(10, 179, 339, 20);
		contentPanel.add(tbCertPath);
		tbCertPath.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse");
		btnBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbCertPath.setText(getOpenFileName());
			}
		});
		btnBrowse.setBounds(361, 178, 91, 23);
		contentPanel.add(btnBrowse);
		
		JLabel lblNewLabel = new JLabel("Certificate password:");
		lblNewLabel.setBounds(10, 203, 184, 14);
		contentPanel.add(lblNewLabel);
		
		tbCertPassword = new JPasswordField();
		tbCertPassword.setBounds(10, 219, 200, 20);
		contentPanel.add(tbCertPassword);
		
		JLabel lblProtectionProperties = new JLabel("Protection properties:");
		lblProtectionProperties.setBounds(10, 243, 169, 14);
		contentPanel.add(lblProtectionProperties);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(10, 263, 441, 56);
		contentPanel.add(scrollPane_1);
		
		tbProtProps = new JTextArea();
		scrollPane_1.setViewportView(tbProtProps);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						okClick();
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
	}

	protected void okClick() {
        m_EncryptionCerts.clear();
        TElX509Certificate cert = new TElX509Certificate();
        TElFileStream f = new TElFileStream(tbCertPath.getText(), "rw", true);
        try
        {
            int r = cert.loadFromStreamPFX(f, new String(tbCertPassword.getPassword()), 0);
            if (r != 0)
            {
                showErrorMessage("PFX read error " + r);
                return;
            }
            m_EncryptionCerts.add(cert, true);
        }
        finally
        {
            f.Free();
        }
        m_Handler.setEncryptionCertificates(m_EncryptionCerts);
        if (!m_Handler.canDecrypt())
        {
            showMessage("Handler is not ready for decryption. Please provide the necessary certificate along with the corresponding private key.");
        }
	}

	public void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	public void showErrorMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
	}

	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

		return "";
	} 

	public void init(TElDefaultDataStorageSecurityHandler handler) {
        m_Handler = handler;
        tbEncryptionCerts.setText("");
        String S = "";
        for (int i = 0; i < handler.getCertIDCount(); i++) 
        {
            S = "Issuer: " + handler.getCertID(i).getIssuer().saveToDNString() + ", serial number: " + SBUtils.binaryToString(handler.getCertID(i).getSerialNumber());
            tbEncryptionCerts.setText(tbEncryptionCerts.getText() + S + "\r\n");
        }
        m_EncryptionCerts.clear();
        S = "";
        S = S + "Data encryption algorithm: " + SBConstants.getAlgorithmNameByAlgorithm(handler.getDataEncryptionAlgorithm()) + "\r\n";
        S = S + "Key encryption algorithm: " + SBConstants.getAlgorithmNameByAlgorithm(handler.getKeyEncryptionAlgorithm()) + "\r\n";
        if (handler.getAuthInfoType() == TSBDSDataAuthInfoType.aitNone) 
        {
            S = S + "Data authentication: none" + "\r\n";
        } else if (handler.getAuthInfoType() == TSBDSDataAuthInfoType.aitBasic) 
        {
            S = S + "Data authentication: basic" + "\r\n";
            S = S + "Digest algorithm: " + SBConstants.getAlgorithmNameByAlgorithm(handler.getAuthDigestAlgorithm()) + "\r\n";
        } else if (handler.getAuthInfoType() == TSBDSDataAuthInfoType.aitExtended) 
        {
            S = S + "Data authentication: extended" + "\r\n";
            if (handler.getAuthDigestLocation() == TSBDSDigestLocation.dlLocal) {
                S = S + "Digest location: embedded to metadata";
            } else if (handler.getAuthDigestLocation() == TSBDSDigestLocation.dlAppended) {
                S = S + "Digest location: appended to data";
            } else if (handler.getAuthDigestLocation() == TSBDSDigestLocation.dlEmbedded) {
                S = S + "Digest location: embedded to data";
            } else {
                S = S + "Digest location: unknown";
            }
            S = S + "\r\n";
            S = S + "Digest algorithm: " + SBConstants.getAlgorithmNameByAlgorithm(handler.getAuthDigestAlgorithm()) + "\r\n";
            S = S + "Block size: " + handler.getAuthBlockSize() + "\r\n";
            S = S + "Digest size: " + handler.getAuthHashSize() + "\r\n";
            S = S + "MAC key length: " + handler.getAuthKeyLen() + "\r\n";
        }
        tbProtProps.setText(S);
	}
}
