package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPCertRetriever;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.LDAP.SBLDAPCRL;
import SecureBlackbox.LDAP.SBLDAPCertRetriever;
import SecureBlackbox.PDF.*;
import SecureBlackbox.PKIPDF.SBPAdES;
import SecureBlackbox.PKIPDF.TElPDFAdvancedPublicKeySecurityHandler;
import SecureBlackbox.PKIPDF.TSBPAdESSignatureType;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@SuppressWarnings("serial")
public class SelectServlet extends HttpServlet {	
	
	static {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");
	
        SBPDF.initialize();
        SBPAdES.initialize();
        SBPDFSecurity.initialize();

        SBHTTPCRL.registerHTTPCRLRetrieverFactory();
        SBLDAPCRL.registerLDAPCRLRetrieverFactory();
        SBHTTPOCSPClient.registerHTTPOCSPClientFactory();
        SBHTTPCertRetriever.registerHTTPCertificateRetrieverFactory();
        SBLDAPCertRetriever.registerLDAPCertificateRetrieverFactory();
	}

    private Cache cache = Cache.getInstance();
	
	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String type = req.getParameter("type");
		String ades = req.getParameter("ades");
		boolean createAdES = "yes".equals(ades);
		
		// temporary copy of the document
		
		byte[] docBin = Utils.getPDFFile(this);
		TElMemoryStream doc = new TElMemoryStream(docBin, 0, docBin.length);
		TElMemoryStream output = new TElMemoryStream();
		
		// prepare
		
		TElDCAsyncState state;

        TElPDFDocument pdf = new TElPDFDocument();
        TElPDFSecurityHandler handler;
        TElPDFSignature signature;
        
        pdf.open(doc);
        
        if (createAdES) {
        	handler = new TElPDFAdvancedPublicKeySecurityHandler();
        	
        	((TElPDFAdvancedPublicKeySecurityHandler)handler).setPAdESSignatureType(TSBPAdESSignatureType.pastEnhanced);
            ((TElPDFAdvancedPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
            ((TElPDFAdvancedPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
        } else {
        	handler = new TElPDFPublicKeySecurityHandler();
        	
        	((TElPDFPublicKeySecurityHandler)handler).setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);
            ((TElPDFPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
            ((TElPDFPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
        }
        
        signature = pdf.getSignatureEntry(pdf.addSignature());
        signature.setSignatureType(SBPDF.stDocument);
        signature.setInvisible(true);
        signature.setHandler(handler);
        signature.setSigningTime(SBUtils.utcNow());
        signature.setExtraSpace(500 * 1024); // if "EElPDFError: No enough space for signature" occurs

        if (createAdES)
            state = pdf.initiateAsyncOperation(TSBDCAsyncSignMethod.asmPKCS7);
        else
            state = pdf.initiateAsyncOperation();

        state.saveToStream(output, SBDCXMLEnc.dcxmlEncoding());

        cache.put(req.getSession().getId(), doc.getBuffer());
        
		// ------------------------------------------ //

        req.setAttribute("paramData", SBEncoding.base64EncodeArray(output.getBuffer(), false));
        req.setAttribute("paramID", req.getSession().getId());
        try {
            if ("activex".equals(type))
                req.getRequestDispatcher("/activex.jsp").forward(req, resp);
            else if ("flex".equals(type))
                req.getRequestDispatcher("/flex.jsp").forward(req, resp);
            else if ("java".equals(type))
                req.getRequestDispatcher("/java.jsp").forward(req, resp);
            else
                resp.sendError(400);
        } catch (ServletException e) {
            resp.sendError(500);
        }
	}
}
