package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPCertRetriever;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.LDAP.SBLDAPCRL;
import SecureBlackbox.LDAP.SBLDAPCertRetriever;
import SecureBlackbox.PDF.*;
import SecureBlackbox.PKIPDF.SBPAdES;
import SecureBlackbox.PKIPDF.TElPDFAdvancedPublicKeySecurityHandler;
import SecureBlackbox.PKIPDF.TSBPAdESSignatureType;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;

@SuppressWarnings("serial")
public class SelectServlet extends HttpServlet {	
	
	static {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");
	
        SBPDF.initialize();
        SBPAdES.initialize();
        SBPDFSecurity.initialize();

        SBHTTPCRL.registerHTTPCRLRetrieverFactory();
        SBLDAPCRL.registerLDAPCRLRetrieverFactory();
        SBHTTPOCSPClient.registerHTTPOCSPClientFactory();
        SBHTTPCertRetriever.registerHTTPCertificateRetrieverFactory();
        SBLDAPCertRetriever.registerLDAPCertificateRetrieverFactory();
	}

    private Cache cache = Cache.getInstance();
	
	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String type = req.getParameter("type");
		String ades = req.getParameter("ades");
		boolean createPAdES = "yes".equals(ades);		
		
		TElMemoryStream output = new TElMemoryStream();

		// prepare
		TElDCAsyncStateCollection States = new TElDCAsyncStateCollection(true);
		try
		{
			presignDocument(req, "sample.pdf", createPAdES, States);
			presignDocument(req, "sample2.pdf", createPAdES, States);
			
	        States.saveToStream(output, SBDCXMLEnc.dcxmlEncoding());
		}
		finally
		{
			States.Free();
		}

		// ------------------------------------------ //

        req.setAttribute("paramData", SBEncoding.base64EncodeArray(output.getBuffer(), false));
        req.setAttribute("paramID", req.getSession().getId());
        try {
            if ("activex".equals(type))
                req.getRequestDispatcher("/activex.jsp").forward(req, resp);
            else if ("flex".equals(type))
                req.getRequestDispatcher("/flex.jsp").forward(req, resp);
            else if ("java".equals(type))
                req.getRequestDispatcher("/java.jsp").forward(req, resp);
            else
                resp.sendError(400);
        } catch (ServletException e) {
            resp.sendError(500);
        }
	}
  
	  private void presignDocument(HttpServletRequest req, String filename, Boolean createPAdES, TElDCAsyncStateCollection states)
	  {
		  // temporary copy of the document
		  byte[] docBin = Utils.getPDFFile(this, filename);
		  TElMemoryStream stream = new TElMemoryStream(docBin, 0, docBin.length);
	    
		  TElDCAsyncState state;

		  TElPDFDocument Document = new TElPDFDocument();
		  TElPDFSecurityHandler handler;
		  TElPDFSignature signature;
		  try
		  {     
			  Document.open(stream);
	        
			  if (createPAdES) {
				  handler = new TElPDFAdvancedPublicKeySecurityHandler();
	        	
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setPAdESSignatureType(TSBPAdESSignatureType.pastEnhanced);
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
			  } else {
				  handler = new TElPDFPublicKeySecurityHandler();
	        	
				  ((TElPDFPublicKeySecurityHandler)handler).setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);
				  ((TElPDFPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
				  ((TElPDFPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
			  }
	        
			  signature = Document.getSignatureEntry(Document.addSignature());
			  signature.setSignatureType(SBPDF.stDocument);
			  signature.setInvisible(true);
			  signature.setHandler(handler);
			  signature.setSigningTime(SBUtils.utcNow());
			  //signature.setExtraSpace(500 * 1024); // if "EElPDFError: No enough space for signature" occurs    

			  if (createPAdES)
				  state = Document.initiateAsyncOperation(TSBDCAsyncSignMethod.asmPKCS7);
			  else
				  state = Document.initiateAsyncOperation();
			  
			  states.add(state, false);

			  cache.put(req.getSession().getId() + "." + Integer.toString(states.getCount()), stream.getBuffer());       
		  }
		  finally
		  {
			  handler.Free();
			  Document.Free();
			  stream.Free();
		  }        
	}
}
