package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPCertRetriever;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.LDAP.SBLDAPCRL;
import SecureBlackbox.LDAP.SBLDAPCertRetriever;
import SecureBlackbox.XML.SBXMLDefs;
import SecureBlackbox.XML.TElXMLDOMDocument;
import SecureBlackbox.XMLSecurity.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@SuppressWarnings("serial")
public class SelectServlet extends HttpServlet {	
	
	static {
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");


        SBHTTPCRL.registerHTTPCRLRetrieverFactory();
        SBLDAPCRL.registerLDAPCRLRetrieverFactory();
        SBHTTPOCSPClient.registerHTTPOCSPClientFactory();
        SBHTTPCertRetriever.registerHTTPCertificateRetrieverFactory();
        SBLDAPCertRetriever.registerLDAPCertificateRetrieverFactory();
	}

    private Cache cache = Cache.getInstance();
	
	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String type = req.getParameter("type");
		String ades = req.getParameter("ades");
		boolean createAdES = "yes".equals(ades);
		
		// temporary copy of the document
		
		byte[] docBin = Utils.getXMLFile(this);
		TElMemoryStream doc = new TElMemoryStream(docBin, 0, docBin.length);
		TElMemoryStream output = new TElMemoryStream();
		
		// prepare
		
		TElDCAsyncState state;

        TElXMLDOMDocument xml = new TElXMLDOMDocument();
        xml.loadFromStream(doc);

        TElXMLSigner signer = new TElXMLSigner();
        signer.setSignatureType(SBXMLSec.xstEnveloped);
        signer.setSignatureMethodType(SBXMLSec.xmtSig);
        signer.setSignatureMethod(SBXMLSec.xsmRSA_SHA1);
        signer.setIncludeKey(true);

        if (createAdES) {
            TElXAdESSigner xadesSigner = new TElXAdESSigner();
            signer.setXAdESProcessor(xadesSigner);

            xadesSigner.setXAdESVersion(SBXMLAdES.XAdES_v1_3_2);
            xadesSigner.setSigningTime(SBUtils.utcNow());

            xadesSigner.generate();
            xadesSigner.getQualifyingProperties().setXAdESPrefix("xades");
        }

        TElXMLReference ref = new TElXMLReference();
        ref.setDigestMethod(SBXMLSec.xdmSHA1);
        ref.setURINode(xml.getDocumentElement());
        ref.setURI("");
        ref.getTransformChain().add(new TElXMLEnvelopedSignatureTransform());
        signer.getReferences().add(ref);

        signer.updateReferencesDigest();
        signer.generateSignatureAsync();

        TSBObject var = new TSBObject();
        var.value = xml.getDocumentElement();

        state = signer.initiateAsyncSign(var);
        state.saveToStream(output, SBDCXMLEnc.dcxmlEncoding());

        doc.setLength(0);
        xml.saveToStream(doc, SBXMLDefs.xcmNone, "");

        cache.put(req.getSession().getId(), doc.getBuffer());
        
		// ------------------------------------------ //

        req.setAttribute("paramData", SBEncoding.base64EncodeArray(output.getBuffer(), false));
        req.setAttribute("paramID", req.getSession().getId());
        try {
            if ("activex".equals(type))
                req.getRequestDispatcher("/activex.jsp").forward(req, resp);
            else if ("flex".equals(type))
                req.getRequestDispatcher("/flex.jsp").forward(req, resp);
            else if ("java".equals(type))
                req.getRequestDispatcher("/java.jsp").forward(req, resp);
            else
                resp.sendError(400);
        } catch (ServletException e) {
            resp.sendError(500);
        }
	}
}
