import SecureBlackbox.Base.*;

public class ServerThread
{
	private Logger m_logger;
	private TcpListener m_tcpListener;
	private java.lang.Thread m_listenThread;

	private TElCustomCertStorage m_certStorage;
	private boolean m_sslMode;
	private boolean m_AuthDigest;
	private boolean m_AuthBasic;
	private int m_port;
	private String m_basePath;
	
	private boolean m_disposed;

	private ArrayList m_list;
	private TElSharedResource m_mutex;

	public ServerThread(Logger logger, int Port, boolean SSLMode, TElCustomCertStorage CertStorage,
		String BasePath, boolean AuthBasic, boolean AuthDigest)
	{
		m_logger = logger;
		m_sslMode = SSLMode;
		m_certStorage = CertStorage;
		m_port = Port;
		m_basePath = BasePath;
		m_AuthBasic = AuthBasic;
		m_AuthDigest = AuthDigest;

		m_list = new ArrayList();
		m_mutex = new TElSharedResource();

		m_listenThread = new java.lang.Thread(new Runnable() {
			
			public void run() {
				ListenForClients();				
			}
		});
		
		m_listenThread.start();
	}

	public void Dispose()
	{
		if (! m_disposed)
		{
			m_tcpListener.stop();
			m_disposed = true;

			while (m_list.getCount() > 0)
			{
				((ClientThread)m_list.get(0)).Dispose();
			}
		}
	}

	public void AddConnection(ClientThread clientThread)
	{
		m_mutex.waitToWrite();
		try
		{
			m_list.add(clientThread);
		}
		finally
		{
			m_mutex.done();
		}
	}

	public void RemoveConnection(ClientThread clientThread)
	{
		m_mutex.waitToWrite();
		try
		{
			m_list.remove(clientThread);
		}
		finally
		{
			m_mutex.done();
		}
	}

	private void ListenForClients()
	{
		m_tcpListener = new TcpListener("127.0.0.1", m_port, this) {
			
			@Override
			void started(TElSocket socket) {
				Logger.Log("TcpListener was started.");
			}
			
			@Override
			void connected(final TElSocket socket, final ServerThread sender) {
					
				java.lang.Thread job = new java.lang.Thread(new Runnable() {
					public void run() {
						TcpClient client = new TcpClient(socket);
						ClientThread clientThread = new ClientThread(sender, client, m_certStorage, 
								m_sslMode, m_logger, m_basePath, m_AuthBasic, m_AuthDigest);
						clientThread.Execute();						
					}
				});

				job.start();
			}
		};
		
		m_tcpListener.start();
		m_tcpListener.accept();
	}
}
