

import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.LDAP.*;
import SecureBlackbox.SSLCommon.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JPasswordField;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTextField tbServer;
	private JTextField tbRootDN;
	private JTextField tbPort;
	private JPasswordField tbPassword;
	private JTextField tbBaseDN;
	private JTable table;
	private JComboBox<String> cmbScope;
	private JButton btnLoad;
	private JButton btnAdd;
	private JButton btnRemove;
	private TElLDAPSClient LDAPSClient;
	private TElLDAPCertStorage LDAPCertStorage;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setTitle("EldoS LDAP Certificate Storage Demo");
		setBounds(100, 100, 589, 302);
		getContentPane().setLayout(null);
		
		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(null, "Login info", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel.setBounds(10, 11, 561, 85);
		getContentPane().add(panel);
		panel.setLayout(null);
		
		JLabel lblLdapServer = new JLabel("LDAP server");
		lblLdapServer.setHorizontalAlignment(SwingConstants.RIGHT);
		lblLdapServer.setBounds(10, 23, 85, 14);
		panel.add(lblLdapServer);
		
		JLabel lblRootDn = new JLabel("Root DN");
		lblRootDn.setHorizontalAlignment(SwingConstants.RIGHT);
		lblRootDn.setBounds(10, 48, 85, 14);
		panel.add(lblRootDn);
		
		JLabel lblPort = new JLabel("Port");
		lblPort.setHorizontalAlignment(SwingConstants.RIGHT);
		lblPort.setBounds(231, 23, 85, 14);
		panel.add(lblPort);
		
		JLabel lblPasswor = new JLabel("Password");
		lblPasswor.setHorizontalAlignment(SwingConstants.RIGHT);
		lblPasswor.setBounds(231, 48, 85, 14);
		panel.add(lblPasswor);
		
		final JCheckBox cbUseSSL = new JCheckBox("Use SSL");
		cbUseSSL.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				LDAPSClient.setEnabled(cbUseSSL.isSelected());
				if (cbUseSSL.isSelected())
					tbPort.setText("636");
				else
					tbPort.setText("389");
			}
		});
		cbUseSSL.setBounds(447, 19, 97, 23);
		panel.add(cbUseSSL);
		
		tbServer = new JTextField();
		tbServer.setBounds(105, 20, 116, 20);
		panel.add(tbServer);
		tbServer.setColumns(10);
		
		tbRootDN = new JTextField();
		tbRootDN.setColumns(10);
		tbRootDN.setBounds(105, 45, 116, 20);
		panel.add(tbRootDN);
		
		tbPort = new JTextField();
		tbPort.setHorizontalAlignment(SwingConstants.RIGHT);
		tbPort.setText("389");
		tbPort.setColumns(10);
		tbPort.setBounds(325, 20, 116, 20);
		panel.add(tbPort);
		
		tbPassword = new JPasswordField();
		tbPassword.setColumns(10);
		tbPassword.setBounds(325, 45, 116, 20);
		panel.add(tbPassword);
		
		JLabel lblBaseDn = new JLabel("Base DN");
		lblBaseDn.setHorizontalAlignment(SwingConstants.RIGHT);
		lblBaseDn.setBounds(10, 107, 60, 14);
		getContentPane().add(lblBaseDn);
		
		JLabel lblScope = new JLabel("Scope");
		lblScope.setHorizontalAlignment(SwingConstants.RIGHT);
		lblScope.setBounds(10, 138, 60, 14);
		getContentPane().add(lblScope);
		
		tbBaseDN = new JTextField();
		tbBaseDN.setBounds(80, 104, 195, 20);
		getContentPane().add(tbBaseDN);
		tbBaseDN.setColumns(10);
		
		cmbScope = new JComboBox<String>();
		cmbScope.setModel(new DefaultComboBoxModel<String>(new String[] {"Base", "One Level", "Sub Tree"}));
		cmbScope.setBounds(80, 134, 195, 22);
		getContentPane().add(cmbScope);
		
		btnLoad = new JButton("Load");
		btnLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try	{
					
					loadCertClick();
					
				} catch (EElLDAPSClientError Ex) {
					showErrorMessage(Ex.getMessage() + " [Error code: " + Ex.getErrorCode() + "]", "Error");
				}
			}
		});
		btnLoad.setBounds(299, 117, 76, 23);
		getContentPane().add(btnLoad);
		
		btnAdd = new JButton("Add");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					
					addClick();
				
				} catch (EElLDAPSClientError Ex) {
					showErrorMessage(Ex.getMessage() + " [Error code: " + Ex.getErrorCode() + "]", "Error");
				}
			}
		});
		btnAdd.setBounds(386, 117, 76, 23);
		getContentPane().add(btnAdd);
		
		btnRemove = new JButton("Remove");
		btnRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					
					removeClick();
					
				} catch (EElLDAPSClientError Ex) {
					showErrorMessage(Ex.getMessage() + " [Error code: " + Ex.getErrorCode() + "]", "Error");
				}				
			}
		});
		btnRemove.setBounds(472, 117, 84, 23);
		getContentPane().add(btnRemove);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(0, 166, 581, 107);
		getContentPane().add(scrollPane);
		
		table = new JTable(new CertTableModel());
		table.setFillsViewportHeight(true);
		scrollPane.setViewportView(table);

		init();
	}

	protected void loadCertClick() throws EElLDAPSClientError {
		LDAPSClient.setAddress(tbServer.getText());
		LDAPSClient.setPort(Integer.parseInt(tbPort.getText()));
		LDAPSClient.setLDAPDN(tbRootDN.getText());
		LDAPSClient.setPassword(new String(tbPassword.getPassword()));
		LDAPSClient.setBaseDN(tbBaseDN.getText());
		LDAPSClient.setScope(TSBLDAPScope.scSubTree);

		String scope = (String)cmbScope.getSelectedItem();
		if (scope.compareTo("Base") == 0)
		{
			LDAPSClient.setScope(TSBLDAPScope.scBase);
		}
		else if (scope.compareTo("One Level") == 0)
		{
			LDAPSClient.setScope(TSBLDAPScope.scOneLevel);
		}
		else if (scope.compareTo("Sub Tree") == 0)
		{
			LDAPSClient.setScope(TSBLDAPScope.scSubTree);
		}

		LoadCertificates();
	}

	private void showErrorMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);		
	}

	protected void removeClick() throws EElLDAPSClientError {
		int[] rowIndexes = table.getSelectedRows();
		
		for (int i = 0; i < rowIndexes.length; i++)
			LDAPCertStorage.remove(rowIndexes[i]);

		LoadCertificates();		
	}

	protected void addClick() throws EElLDAPSClientError {
		String fileName = getFileName();
		if (fileName.length() > 0)
		{
			TElX509Certificate Cert = new TElX509Certificate();

			if (Cert.loadFromFileAuto(fileName, "") == 0)
				LDAPCertStorage.add(Cert, false);

			LoadCertificates();
		}
	}

	String getFileName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
		
		LDAPSClient = new TElLDAPSClient();
		LDAPSClient.setOnCertificateValidate(new TSBCertificateValidateEvent(DoCertificateValidate));

		LDAPCertStorage = new TElLDAPCertStorage();
		LDAPCertStorage.setLDAPSClient(LDAPSClient);		
	}
	
	TSBCertificateValidateEvent.Callback DoCertificateValidate = new TSBCertificateValidateEvent.Callback() {
		
		public void tsbCertificateValidateEventCallback(TObject sender,
				TElX509Certificate cert, TSBBoolean accept) {
			accept.value = true;
		}
	};
	
	private void LoadCertificates()
	{
		try
		{
			clearCertificates();

			for (int i = 0; i < LDAPCertStorage.getCount(); i++)			
			{
				TElX509Certificate ldapCert = LDAPCertStorage.getCertificate(i);
				String n = SBUtils.binaryToString(ldapCert.getSerialNumber());
				String in = ldapCert.getIssuerName().CommonName;
				String sn = ldapCert.getSubjectName().CommonName;
				
				CertTableModel model = (CertTableModel)table.getModel();
				model.addRow(n, in, sn);
			}
		}
		finally
		{
		}
	}

	private void clearCertificates() {
		CertTableModel model = (CertTableModel)table.getModel();
		model.clear();
	}

}
