import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.tree.DefaultMutableTreeNode;

import SecureBlackbox.MIME.*;

public class ElMimeParserTask extends Object
{
	 DefaultMutableTreeNode fParent = null;
	 String fFileName = "";
	 TAnsiStringMIMEStream fDataStream = null;
	 String fDefaultHeaderCharset = "";
	 String fDefaultBodyCharset = "";
	 boolean fDefaultActivatePartHandlers = false;
	 String fErrorMsg = "";
	 ElMessageDemo fMsg = null;
	 Exception fErrorException = null;
	 boolean fUseBackgroundParser = false;
	 TreeNodeInfo fNode = null;
	 Date fStartTime = new Date();

	 private boolean fFreeOnTerminate = false;

	 public boolean getFreeOnTerminate() {
	     return fFreeOnTerminate;
	 }
	
	 public void setFreeOnTerminate(boolean value) {
	     fFreeOnTerminate = value;
	 }

	 public ElMimeParserTask(DefaultMutableTreeNode aParent, String aFileName,
			 TAnsiStringMIMEStream aDataStream) throws Exception
	 {
	   fParent = aParent;
	   fFileName = aFileName;
	   fDataStream = aDataStream;
	
	   fDefaultHeaderCharset = ProjectOptions.fDefaultHeaderCharset;
	   fDefaultBodyCharset = ProjectOptions.fDefaultBodyCharset;
	   fDefaultActivatePartHandlers = ProjectOptions.fDefaultActivatePartHandlers;
	   fUseBackgroundParser = ProjectOptions.fUseBackgroundParser;
	
	   fFreeOnTerminate = false;
	
	   if ( fUseBackgroundParser) {
	     Resume();
	   }
	   else
	   {
	     try
	     {
	       Execute();
	     }
	     finally
	     {
	     }
	   }
	 }

	 private static String[] headerAddressesFields =
	 {
	   "From",
	   "To",
	   "Reply-To",
	   "Sender",
	   "CC",
	   "BCC",
	   "Resent-From",
	   "Resent-To",
	   "Resent-Reply-To",
	   "Resent-Sender",
	   "Resent-CC",
	   "Resent-BCC"
	 };
     
	 protected void Execute() throws Exception
	 {
	   try
	   {
	     fMsg = new ElMessageDemo();
	
         AddMessageToItems();
	
	     // initialize fields
	
	     fMsg.fUseBackgroundParser = fUseBackgroundParser;
	     fMsg.fParserThread = this;
	
	     if (fDataStream == null) {
	       fMsg.fDataFile = fFileName;
	       fMsg.fDataStream.loadFromFile(fMsg.fDataFile);
	     }
	     else
	     {
	    	 byte[] data = fDataStream.getMemory();
	         fMsg.fDataStream.setMemory(data, data.length);
	     }
	     fStartTime = new Date();
	
	     short k = SBMIME.mpoStoreStream |
		           SBMIME.mpoLoadData |
		           SBMIME.mpoCalcDataSize;
	     fMsg.fResult =
	       fMsg.parseMessage(
	         fMsg.fDataStream,
	         fDefaultHeaderCharset,
	         fDefaultBodyCharset,
	         k,
	         false,
	         false,
	         fDefaultActivatePartHandlers
	     );
	
	     if ( (fMsg.fResult == SBMIME.EL_OK) ||
	       (fMsg.fResult == SBMIME.EL_WARNING) )
	     {
	      // parse rfc header fields to e-mail AddressList collections:
	
	       fMsg.InitMailAdressFields(headerAddressesFields);
	     }
	
		 int fres = fMsg.fResult;
	
	     if ( ! ( ( (fMsg.fResult == SBMIME.EL_OK) || (fMsg.fResult == SBMIME.EL_WARNING) ))) {
	       fErrorMsg = String.format(
	         "Error parsing mime/smime message \"%s\". ElMime error code: %d",
	         fFileName, fres
	       );
	     }
	
       UnlinkMessage();
	
	   }
	   catch (Exception e) {
	     fErrorMsg = e.getMessage();
	     fErrorException = e;
	     if ( fUseBackgroundParser ) {
	       UnlinkMessage();
	       ShowError();
	     }
	     else
	     {
	       UnlinkMessage();
	       ShowError();
	     }
	   }
	 }

	 protected void Resume() throws Exception
	 {
	   // todo: ???
	   throw new Exception("Background thread parser not implemented");
	 }
	
	 public void Terminate()
	 {
	   //fTerminated = true;
	   // todo: ???
	 }
	
	 public void WaitFor()
	 {
	   // todo: ???
	 }

	 public void Dispose()
	 {
	   if ( fDataStream != null) {
		   fDataStream.Free();
		  fDataStream = null;
	   }
	 }

	 private synchronized void ShowError() throws Exception
	 {
	   throw fErrorException; //???
	 }

	 public static String ExtractFileName(String fileName)
	 {
	   String sFileName = ""+fileName;
	   for ( int i = sFileName.length()-1; i>=0; i--) {
	     if ( (sFileName.charAt(i) == '\\') || (sFileName.charAt(i) == '/') )
	     {
	    	 StringBuilder sb = new StringBuilder(sFileName);
	    	 sb.delete(0, i);
	    	 return sb.toString();
	     }
	   };
	   return sFileName;
	 }

 private synchronized void AddMessageToItems()
 {
   TreeNodeInfo NullNode = null;
   String S = "";

   if (fNode == null) {
   // BEFORE PARSE
     //S = fStartTime.ToString("hh:nn:ss.zzz", null);
     S = fStartTime.toString();
     if (fFileName.length() > 0)
       S += " | \"" + ExtractFileName(fFileName) + "\"";
     else
       S += " | \"//Attached Message//\"";
     if (fUseBackgroundParser)
       S = "...wait..." + S;
     fNode = new TreeNodeInfo(TagInfo.tiParsedMessage, fMsg);
     fNode.setText(S);
     fNode.setUserObject(S);
     //fNode.ImageIndex := fNode.ImageIndex-1;
     //fNode.SelectedIndex := fNode.ImageIndex;
     fNode.fLocked = true; // do not allow remove fro TreeView
     fParent.add(fNode);
   }
   else
   {
   // AFTER PARSE
     //fNode.ImageIndex := fNode.ImageIndex+1;
     //fNode.SelectedIndex := fNode.ImageIndex;
     //S = fStartTime.ToString("hh:nn:ss.zzz", null);
     S = new Date(subtractDays(new Date(), fStartTime)).toString();
     if (fFileName.length() > 0)
       S = "[ " + S + " ] \"" + ExtractFileName(fFileName) + "\"";
     else
       S = "[ " + S + " ] \"//Attached Message//\"";
     fNode.setText(S);
     fNode.setUserObject(S);
     //if (fErrorMsg.Length == 0)
     {
       NullNode = new TreeNodeInfo(TagInfo.tiNull, null);
       NullNode.setText("...");
       NullNode.setUserObject("...");
       fNode.add(NullNode);
     }
     if (fErrorMsg.length() > 0)
     {
       NullNode = new TreeNodeInfo(TagInfo.tiError, null);
       NullNode.setText(fErrorMsg);
       NullNode.setUserObject(fErrorMsg);
       fNode.add(NullNode);
     }

     fNode.fLocked = false; // allow remove from TreeView
   }
   //if ( (fNode != null)/* && (fParent.Parent == null)*/ )
     //fParent.expand();

 }

 public int subtractDays(Date date1, Date date2)
 {
   GregorianCalendar gc1 = new GregorianCalendar();  gc1.setTime(date1);
   GregorianCalendar gc2 = new GregorianCalendar();  gc2.setTime(date2);

   int days1 = 0;
   int days2 = 0;
   int maxYear = Math.max(gc1.get(Calendar.YEAR), gc2.get(Calendar.YEAR));

   GregorianCalendar gctmp = (GregorianCalendar) gc1.clone();
   for (int f = gctmp.get(Calendar.YEAR);  f < maxYear;  f++)
     {days1 += gctmp.getActualMaximum(Calendar.DAY_OF_YEAR);  gctmp.add(Calendar.YEAR, 1);}

   gctmp = (GregorianCalendar) gc2.clone();
   for (int f = gctmp.get(Calendar.YEAR);  f < maxYear;  f++)
     {days2 += gctmp.getActualMaximum(Calendar.DAY_OF_YEAR);  gctmp.add(Calendar.YEAR, 1);}

   days1 += gc1.get(Calendar.DAY_OF_YEAR) - 1;
   days2 += gc2.get(Calendar.DAY_OF_YEAR) - 1;

   return (days1 - days2);
 } 
 
 private synchronized void UnlinkMessage()
 {
   fFreeOnTerminate = true;
   if (fMsg != null) {
     if ( fDefaultActivatePartHandlers )
       Dispose();

     fMsg.fParserThread = null;
   }
   if ( fNode != null )
     AddMessageToItems();
 }


}
