import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import SecureBlackbox.Base.TElCertificateRevocationList;
import SecureBlackbox.Base.TElFileStream;
import SecureBlackbox.Base.TElOCSPResponse;
import SecureBlackbox.Base.TElX509Certificate;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class RevInfoDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTable lvItems;
	private boolean isOk;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			RevInfoDialog dialog = new RevInfoDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public RevInfoDialog() {
		setTitle("Revocation information request");
		setBounds(100, 100, 662, 382);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblPleaseSpecifyRevocation = new JLabel("<html><body>Please specify revocation information to be added to the signature. Revocation information consists of (a) the complete certificate chain up to the root CA certificate, (b) CRLs and (c) OCSP responses for each certificate from the chain.</body></html>");
		lblPleaseSpecifyRevocation.setBounds(10, 0, 636, 55);
		contentPanel.add(lblPleaseSpecifyRevocation);
		
		JLabel lblRevocationItems = new JLabel("Revocation items:");
		lblRevocationItems.setBounds(10, 52, 297, 14);
		contentPanel.add(lblRevocationItems);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 77, 536, 214);
		contentPanel.add(scrollPane);
		
		lvItems = new JTable(new PropsListModel("Title", "Item type"));
		lvItems.setFillsViewportHeight(true);
		scrollPane.setViewportView(lvItems);
		
		JButton btnAdd = new JButton("Add");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addClick();
			}
		});
		btnAdd.setBounds(556, 75, 91, 23);
		contentPanel.add(btnAdd);
		
		JButton btnRemove = new JButton("Remove");
		btnRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeClick();
			}
		});
		btnRemove.setBounds(556, 109, 91, 23);
		contentPanel.add(btnRemove);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		Reset();
	}
	
	public void Reset()
	{
		for (int i = 0; i < lvItems.getRowCount(); i++)
		{
			Object tag = ((PropsListModel)lvItems.getModel()).getItemTag(i);

			if (tag instanceof TElX509Certificate)
				((TElX509Certificate)tag).Free();
			if (tag instanceof TElCertificateRevocationList)
				((TElCertificateRevocationList)tag).Free();
			if (tag instanceof TElOCSPResponse)
				((TElOCSPResponse)tag).Free();
		}

		lvItems.removeAll();
	}


	protected String getOpenFileName() {
		JFileChooser fc = new JFileChooser();
	    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	protected void removeClick() {
		int[] rows = lvItems.getSelectedRows();
		for (int i = 0; i < rows.length; i++)
		{
			int row = rows[i];
			Object tag = ((PropsListModel)lvItems.getModel()).getItemTag(row);

			if (tag instanceof TElX509Certificate)
				((TElX509Certificate)tag).Free();
			if (tag instanceof TElCertificateRevocationList)
				((TElCertificateRevocationList)tag).Free();
			if (tag instanceof TElOCSPResponse)
				((TElOCSPResponse)tag).Free();
			
			lvItems.remove(row);
		}	
	}

	protected void addClick() {
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
		{
			TElFileStream F = new TElFileStream(fileName, "r", true);
			try
			{			
				int filterIndex = getFilterIndex(fileName);
				if (filterIndex == 1)
				{
					TElX509Certificate Cert = new TElX509Certificate();
					try
					{
						Cert.loadFromFileAuto(fileName, requestPassword());
					}
					catch(Exception ex)
					{
						showErrorMessage("Failed to read Certificate");
						Cert.Free();
					}

					((PropsListModel)lvItems.getModel()).addItem(Cert.getSubjectName().CommonName, "Certificate", Cert);
				}
				else if (filterIndex == 2)
				{
					TElCertificateRevocationList Crl = new TElCertificateRevocationList();
					int R = Crl.loadFromStream(F, 0);
					if (R != 0)
					{
						showErrorMessage("Failed to read CRL");
						Crl.Free();
						return;
					}

					String s = "";
					for (int i = 0; i < Crl.getIssuer().getCount(); i++)
						s = s + Crl.getIssuer().getValue(i) + ", ";
					s = s + Crl.getThisUpdate();
					
					((PropsListModel)lvItems.getModel()).addItem(s, "CRL", Crl);					
				}
				else if (filterIndex == 3)
				{
					byte[] Buf = new byte[(int) F.getLength()];
					F.read(Buf, 0, Buf.length);
					TElOCSPResponse Ocsp = new TElOCSPResponse();
					try
					{
						Ocsp.load(Buf, 0, Buf.length);
					}
					catch(Exception ex)
					{
						Ocsp.Free();
						showErrorMessage("Failed to load OCSP response");
						return;
					}

					((PropsListModel)lvItems.getModel()).addItem("OCSP response", "OCSP", Ocsp);
				}
			}
			finally
			{
				F.Free();
			}
		}
	}

	private int getFilterIndex(String certFileName) {
		String ext = extension(certFileName).toLowerCase();
		if (ext.compareTo("pfx") == 0) return 1;
		if (ext.compareTo("crl") == 0) return 2;
		return 3;
	}

	public String extension(String fullPath) {
	    int dot = fullPath.lastIndexOf(".");
	    return fullPath.substring(dot + 1);
	}
	
	private void showErrorMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);						
	}

 	private String requestPassword() {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, "Please enter password", JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}

	public boolean isOK() {
		return isOk;
	}

	public int getItemsCount() {
		return lvItems.getRowCount();
	}

	public Object getItemTag(int i) {
		return ((PropsListModel)lvItems.getModel()).getItemTag(i);
	}
}
