package com.eldos.DAVServletSample;

import java.io.IOException;
import java.io.File;

import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.freepascal.rtl.TObject;
import SecureBlackbox.Base.*;
import SecureBlackbox.DAVCommon.*;
import SecureBlackbox.DAVServer.*;
import SecureBlackbox.DAVEE.*;
import SecureBlackbox.SSLCommon.*;
import SecureBlackbox.HTTPCommon.*;
 
public class DemoWebDAVServlet extends HttpServlet 
{
    // hardcoded for simplicity
    static final String ATTRIBUTES_DATABASE = "c:\\webdav.db";
    static final String STORAGE_DIRECTORY = "c:\\webdav";

    TElWebDAVServer webDavServer;
    TElDiskFileSystemAdapter diskAdapter;
    DAVInputStream inputStream;
    DAVOutputStream outputStream;
    private String attrFilePath;
    private boolean available;
    
    @Override
    public void init() throws ServletException {
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");	
        
        createEmptyAttributesDB(ATTRIBUTES_DATABASE);
        attrFilePath = ATTRIBUTES_DATABASE;
        
        diskAdapter = new TElDiskFileSystemAdapter();
        diskAdapter.setOnWriteFileTag(new TElVFSWriteFileTagEvent(DiskAdapter_OnWriteFileTag));
        diskAdapter.setOnReadFileTag(new TElVFSReadFileTagEvent(DiskAdapter_OnReadFileTag));
        diskAdapter.setOnDeleteFileTag(new TElVFSDeleteFileTagEvent(DiskAdapter_OnDeleteFileTag));
        diskAdapter.setBasePath(STORAGE_DIRECTORY);
    }

    @Override
    protected void service(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException 
    {
        available = true;
        
        inputStream = new DAVInputStream(req);
        outputStream = new DAVOutputStream(resp);
        
        webDavServer = new TElWebDAVServer();
        webDavServer.setFileSystemAdapter(diskAdapter);
        webDavServer.setOnSend(new TSBSendEvent(WebDavServer_OnSend));
        webDavServer.setOnReceive(new TSBReceiveEvent(WebDavServer_OnReceive));
        
        webDavServer.open();
        
        while (available)
            webDavServer.dataAvailable();
        
        webDavServer.close(false);
    }
    
    private boolean createEmptyAttributesDB(String filePath) {
        File attrFile = new File(filePath);
        if (attrFile.exists()) 
            return true;
        
        JIniFile ini = new JIniFile(filePath);
        ini.WriteString("/", "<ns>DAV:</ns><name>displayname</name>", "/");
        ini.WriteString("/", "<ns>DAV:</ns><name>getcontentlanguage</name>", "en");
        ini.WriteString("/", "<ns>DAV:</ns><name>getcontenttype</name>", "httpd/unix-directory");
        ini.WriteString("/", "<ns>DAV:</ns><name>getetag</name>", "5e77061a9b955ac03d9a574a8a68d8a58005949fa4fdd3d9e38dbd263028ffe8");
        ini.WriteString("/", "<ns>DAV:</ns><name>resourcetype</name>", "<collection/>");
        ini.WriteString("/", "<ns>DAV:</ns><name>getcontentlength</name>", "0");
        ini.WriteString("/", "<ns>DAV:</ns><name>creationdate</name>", SBWebDAVCommon.dateTimeToISOTime(new Date()));
        ini.WriteString("/", "<ns>DAV:</ns><name>getlastmodified</name>", TElHTTPUtils.dateTimeToHTTPTime(TElHTTPUtils.class, new Date()));
        ini.UpdateFile();
        
        return true;
    }
    
    private String urlRemoveSlash(String URL)
	{
		if ((URL.length() > 0) && (URL != "/") && (URL.charAt(URL.length() - 1) == '/'))
			return URL.substring(0, URL.length() - 2);
		else
			return URL;
	}
    
    TSBReceiveEvent.Callback WebDavServer_OnReceive = new TSBReceiveEvent.Callback() {
		
		public int tsbReceiveEventCallback(TObject sender, byte[] buffer, int maxSize) {
            int result;
        
            try {
                result = inputStream.read(buffer, 0, maxSize);
                result = (result > 0) ? result : 0;
            } catch (IOException ex) {
                result = 0;
            }
          
            if (result == 0)
                available = false;
                
            return result;
		}
	};
	
	TSBSendEvent.Callback WebDavServer_OnSend = new TSBSendEvent.Callback() {
		
		public void tsbSendEventCallback(TObject sender, byte[] buffer) {
            try {
                outputStream.write(buffer, 0, buffer.length);
            } catch (IOException ex) {
                // do nothing
            }
		}
	};
    
    TElVFSDeleteFileTagEvent.Callback DiskAdapter_OnDeleteFileTag = new TElVFSDeleteFileTagEvent.Callback() {
		
		public int tElVFSDeleteFileTagEventCallback(TObject sender, String Path, String TagName) {
			int ResultCode = 0;
			try
			{
				JIniFile ini = new JIniFile(attrFilePath);
				ini.EraseSection(urlRemoveSlash(Path));
                ini.UpdateFile();
			}
			catch (Exception ex)
			{
				ResultCode = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
			}

			return ResultCode;
		}
	};
	
	TElVFSReadFileTagEvent.Callback DiskAdapter_OnReadFileTag = new TElVFSReadFileTagEvent.Callback() {
		
		public int tElVFSReadFileTagEventCallback(TObject sender, String path, String tagName, TSBString tagValue) {
			int ResultCode = 0;
			try
			{
				JIniFile ini = new JIniFile(attrFilePath);
				tagValue.value = ini.ReadString(urlRemoveSlash(path), tagName, "");
				if (tagValue.value == null || tagValue.value.length() == 0)
					tagValue.value =  "#$$";
			}
			catch (Exception ex)
			{
				ResultCode = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
			}
			if (tagValue.value.compareTo("#$$") == 0)
			{
				ResultCode = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
				tagValue.value = "";
			}
			return ResultCode;
		}
	}; 
	
	TElVFSWriteFileTagEvent.Callback DiskAdapter_OnWriteFileTag = new TElVFSWriteFileTagEvent.Callback() {
		
		public int tElVFSWriteFileTagEventCallback(TObject sender, String path, String tagName, String tagValue) {
			int ResultCode = 0;
			try
			{
				JIniFile ini = new JIniFile(attrFilePath);
				ini.WriteString(urlRemoveSlash(path), tagName, tagValue);
                ini.UpdateFile();
			}
			catch (Exception ex)
			{
				ResultCode = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
			}
			return ResultCode;
		}
	};
}